type Frame = VideoFrame | HTMLCanvasElement | OffscreenCanvas | ImageBitmap | ImageData;
interface VideoFile {
    file: File | Blob;
    type: string;
}
type VideoSource = Frame[] | AsyncIterable<Frame> | MediaStream | VideoFile;
type QualityPreset = 'low' | 'medium' | 'high' | 'lossless';
interface VideoConfig {
    codec?: 'avc' | 'hevc' | 'vp9' | 'vp8' | 'av1';
    bitrate?: number;
    hardwareAcceleration?: 'no-preference' | 'prefer-hardware' | 'prefer-software';
    latencyMode?: 'quality' | 'realtime';
    keyFrameInterval?: number;
}
interface AudioConfig {
    codec?: 'aac' | 'opus';
    bitrate?: number;
    sampleRate?: number;
    channels?: number;
    bitrateMode?: 'constant' | 'variable';
}
interface ProgressInfo {
    percent: number;
    processedFrames: number;
    totalFrames?: number;
    fps: number;
    stage: string;
    estimatedRemainingMs?: number;
}
interface EncodeOptions {
    width?: number;
    height?: number;
    frameRate?: number;
    quality?: QualityPreset;
    video?: VideoConfig;
    audio?: AudioConfig | false;
    container?: 'mp4' | 'webm';
    onProgress?: (progress: ProgressInfo) => void;
    onError?: (error: EncodeError) => void;
}
type EncodeErrorType = 'not-supported' | 'initialization-failed' | 'configuration-error' | 'invalid-input' | 'encoding-failed' | 'video-encoding-error' | 'audio-encoding-error' | 'muxing-failed' | 'cancelled' | 'timeout' | 'worker-error' | 'filesystem-error' | 'unknown';
declare class EncodeError extends Error {
    type: EncodeErrorType;
    cause?: unknown;
    constructor(type: EncodeErrorType, message: string, cause?: unknown);
}

/**
 * コアエンコード関数の実装
 */

/**
 * ビデオエンコードのメイン関数
 *
 * @param source エンコードするビデオソース
 * @param options エンコードオプション
 * @returns エンコードされたバイナリデータ
 */
declare function encode(source: VideoSource, options?: EncodeOptions): Promise<Uint8Array>;

export { encode };
