type QualityPreset = 'low' | 'medium' | 'high' | 'lossless';
interface VideoConfig {
    codec?: 'avc' | 'hevc' | 'vp9' | 'vp8' | 'av1';
    bitrate?: number;
    hardwareAcceleration?: 'no-preference' | 'prefer-hardware' | 'prefer-software';
    latencyMode?: 'quality' | 'realtime';
    keyFrameInterval?: number;
}
interface AudioConfig {
    codec?: 'aac' | 'opus';
    bitrate?: number;
    sampleRate?: number;
    channels?: number;
    bitrateMode?: 'constant' | 'variable';
}
interface ProgressInfo {
    percent: number;
    processedFrames: number;
    totalFrames?: number;
    fps: number;
    stage: string;
    estimatedRemainingMs?: number;
}
interface EncodeOptions {
    width?: number;
    height?: number;
    frameRate?: number;
    quality?: QualityPreset;
    video?: VideoConfig;
    audio?: AudioConfig | false;
    container?: 'mp4' | 'webm';
    onProgress?: (progress: ProgressInfo) => void;
    onError?: (error: EncodeError) => void;
}
type EncodeErrorType = 'not-supported' | 'initialization-failed' | 'configuration-error' | 'invalid-input' | 'encoding-failed' | 'video-encoding-error' | 'audio-encoding-error' | 'muxing-failed' | 'cancelled' | 'timeout' | 'worker-error' | 'filesystem-error' | 'unknown';
declare class EncodeError extends Error {
    type: EncodeErrorType;
    cause?: unknown;
    constructor(type: EncodeErrorType, message: string, cause?: unknown);
}

/**
 * エンコード可能性の検証
 */

/**
 * エンコード可能性の検証
 *
 * @param options エンコードオプション
 * @returns エンコード可能かどうか
 */
declare function canEncode(options?: EncodeOptions): Promise<boolean>;
/**
 * 特定のコーデックとプロファイルでのサポート確認（上級者向け）
 */
declare function canEncodeWithProfile(videoCodec: string, audioCodec?: string, profile?: {
    width: number;
    height: number;
    framerate: number;
    videoBitrate: number;
    audioBitrate?: number;
}): Promise<{
    video: boolean;
    audio: boolean;
    overall: boolean;
}>;

export { canEncode, canEncodeWithProfile };
